// C Source File
// Created 6/16/01; 8:55:59 PM
// ASH Copyright (C) by Greg Dietsche 2001, 2002.

//SearchEngine.c
#define NO_EXIT_SUPPORT
#define NO_CALC_DETECT
#define NO_AMS_CHECK
#define OPTIMIZE_ROM_CALLS

#include <tigcclib.h>

#include "OpenASHdb.h"
#include "SearchEngine.h"

//Looks for the first string in the catalog (database) that contains
//SearchStr as its' beginning characters
char* FindFirst(const char* const SearchStr)
{

	register unsigned int c;
	register void *ptr;

	if(!(ptr=OpenASHdb())) return NULL;
		
	ptr+=*(int*)(ptr+2);	//first word is the offset to the act
	
	while(*(char*)ptr)	//while ptr does not == zero
	{
		if((c=strLike(ptr+1,SearchStr)))
			return ptr+c+1;	//this scares me ... what if heap compression... and it is not decalred as const...
			ptr+=*(char*)ptr;	//increment to the next string
	}

	return NULL;	//didn't find any matches so return NULL
}

//returns a pointer to a list of all possible matches
//using standard zero terminated strings as well as a final 0 to terminate the
//list
//returns NULL on error
void* FindAll(register const char* const SearchStr)
{
	register unsigned int c=0;	//actual number of chars in the buffer
	register unsigned int d=0;	//a temporary variable
	register char *dbptr;	//points to the database
	register char *ptr=malloc(SEARCH_BUFFSIZE);	//the returned variable
	
	if(!(dbptr=OpenASHdb()) || !ptr) return NULL;
		
	dbptr+=*(int*)(dbptr+2);	//first word is the offset to the act
	
//just in case there are no entries
	*ptr=0;
	
	while(*dbptr)	//while dbptr does not == zero
	{
		if((strLike(dbptr+1,SearchStr)))
		{
			c+=*(char *)dbptr-1;	//strlen(dbptr)
			
			if(c>SEARCH_BUFFSIZE-2)	// -1, but ptr[c+1]... ALWAYS STAY within the buffer...
				return ptr;
			
			strcpy(ptr+d,dbptr+1);
			d=c;
			ptr[c]=0;	//zero terminate the list
		}
			dbptr+=*dbptr;	//increment to the next string
	}
	ptr[c+1]=0;
	return ptr;	//didn't find any matches so return NULL
}

//strLike(StrToTestForMatch,StrToMatch)
//all characters in s1 must = len(s1) chars in s2
//returns strlen(s1) or 0 if no match
inline unsigned int strLike(register const char* const s1, register const char* const s2)
{
	register unsigned int x=0;
	
	while(s1[x]==s2[x] && s2[x]!=0)
		x++;
		
	return s2[x]==0?x:0;
}
