// C Source File
// Created 6/22/01; 4:58:36 PM

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization
#include <tigcclib.h>

#include "DataBase.h"
#include "ConvertDB.h"
#include "editdb.h"
#include "Editor.h"
#include "EditorSupport.h"
#include "EditorMenus.h"
#include "tigcc.h"
#include "Resources.h"

/*
typedef struct 
{
	unsigned short len;
	unsigned short dbtag;
	unsigned short act;
	unsigned short aliases;
	unsigned short custommenu;
}IDBHeader;
*/

int Editor_Open(void)
{
	volatile HANDLE *hACT = &ghACT;
	volatile HANDLE *hALIAS = &ghALIAS;
	volatile HANDLE *hCUST = &ghCUST;
	
	void *db;
	char *dbtag[]={"I DB",NULL};
	unsigned char dbtag0[]={0xF8,0};
	HSym sym;
	int i;
	
//save the current work
	if(!Editor_Save())
		return FALSE;

//display the Open file dialog box
	sym=VarOpen(dbtag0, dbtag);
	
	if(sym.folder==0)
		return FALSE;
		
	HSYMtoName(sym, gdbname);

//free the handles if need be
	if(*hACT)HeapFree(*hACT);
	if(*hALIAS)HeapFree(*hALIAS);
	if(*hCUST)HeapFree(*hCUST);
	
	db=OpenDataBase(gdbname);
	*hACT=ConvertDB(db, CONVERT_ACT);
	*hCUST=ConvertDB(db, CONVERT_CUSTOMMENU);
	*hALIAS=ConvertDB(db, CONVERT_ALIASES);
	CloseDataBase(gdbname);
	
	if(!*hACT || !*hALIAS || !*hCUST)
		return FALSE;
	
	for(i=2;i>-1;i--)
		ChangeWindow(i);
	
	return TRUE;
}

int Editor_SaveAs(void)
{
	char *dbtag[]={"I DB",NULL};
	unsigned char dbtag0[]={0xF8,0};
	HSym sym;
	int x;
	
//display the New file dialog box
	sym=VarSaveAs(dbtag0, NULL, dbtag);
	if(sym.folder==0)
		return FALSE;
	HSYMtoName(sym, gdbname);
	HSymDel(sym);	//not really necessary...
	
	x=Editor_Save();

	ChangeWindow(0);
	
	return x;
}


int Editor_Save(void)
{
	volatile HANDLE *hACT = &ghACT;
	volatile HANDLE *hALIAS = &ghALIAS;
	volatile HANDLE *hCUST = &ghCUST;
	
	unsigned char dbtag[]={0,'I',' ','D','B',0,0xF8};
	unsigned int readpos=0;
	unsigned int writepos=0;
	unsigned int allocsize = SAVE_BUFFSIZE;
	unsigned char *ptrin;
	unsigned char *ptrout;
	volatile HANDLE h;
	HSym sym;
	int x=0;
	int c=0;
	
//prepare to save... take as little mem as possible
	TE_shrinkWrap(te_act);
	TE_shrinkWrap(te_alias);
	TE_shrinkWrap(te_cust);
	//HeapCompress();	//make everything go a little smoother

//allocate the initial buffer
	if((ptrout=HeapDeref(h=HeapAlloc(SAVE_BUFFSIZE)))==H_NULL)
		return FALSE;

	((IDBHeader*)ptrout)->dbtag=0xFFFF;	//magic number
	((IDBHeader*)ptrout)->act=sizeof(IDBHeader)-2;
	
	writepos+=sizeof(IDBHeader);
	
//compile ACT
	ptrin=HLock(*hACT);
	x=writepos++;
	
	while(ptrin[readpos])	//the editor is zero terminated
	{
		if(writepos+6>allocsize)
		{
			allocsize+=SAVE_BUFFSIZE;
			if((h=HeapRealloc(h, allocsize))==H_NULL)
				return FALSE;
			ptrout = HeapDeref(h);
		}
				
		ptrout[writepos++]=ptrin[readpos++];	//byte by byte copy
		
		if(ptrout[writepos-1]==0xD)	//if \r then do zero term, length byte
		{
			ptrout[x]=writepos-x;	//write the length byte
			ptrout[writepos-1]=0;	//zero term string
			x=writepos++;					//save the current position for later length byte
		}
	}
	
	ptrout[x]=writepos-x;
	ptrout[writepos++]=0;	//null term the string
	ptrout[writepos++]=0;	//null term act list

	HeapUnlock(*hACT);
	
//compile Custom Menu
	ptrin=HLock(*hCUST);
	readpos=0;
	((IDBHeader*)ptrout)->cust=writepos-2;
	x=writepos++;
		
	while(ptrin[readpos])	//the editor is zero terminated
	{
		if(writepos+6>allocsize)
		{
			allocsize+=SAVE_BUFFSIZE;
			if((h=HeapRealloc(h, allocsize))==H_NULL)
				return FALSE;
			ptrout = HeapDeref(h);
		}
		
		ptrout[writepos++]=ptrin[readpos++];	//byte by byte copy
		
		if(ptrout[writepos-1]==0xD)	//if /r then do zero term, length byte
		{
			ptrout[x]=writepos-x;	//write the length byte
			ptrout[writepos-1]=0;	//zero term string
			x=writepos++;					//save the current position for later length byte
		}
	}
	
	ptrout[x]=writepos-x;
	ptrout[writepos++]=0;	//null term the string
	ptrout[writepos++]=0;	//null term the list

	HeapUnlock(*hCUST);
	
//compile Aliases
	ptrin=HLock(*hALIAS);
	readpos=0;
	((IDBHeader*)ptrout)->alias=writepos-2;
	x=writepos++;
	
	while(ptrin[readpos])	//the editor is zero terminated
	{
		if(writepos+6>allocsize)
		{
			allocsize+=SAVE_BUFFSIZE;
			if((h=HeapRealloc(h,allocsize))==H_NULL)
				return FALSE;
			ptrout = HeapDeref(h);
		}
		
		ptrout[writepos++]=ptrin[readpos++];	//byte by byte copy

		if(ptrout[writepos-1]==0xD)
		{
			c++;
			ptrout[writepos-1]=0;
		}
		
		if(c==2)
		{
			c=0;
			ptrout[x]=writepos-x;	//write the length byte
			ptrout[writepos-1]=0;	//zero term string
			x=writepos++;					//save the current position for later length byte
		}
	}
	
	ptrout[x]=writepos-x;
	ptrout[writepos++]=0;	//null term the string
	ptrout[writepos++]=0;	//null term the list
	
	HeapUnlock(*hALIAS);
	
	//////////////////////////
	
	
//write the database tag
		if(writepos+4+sizeof(dbtag)>allocsize)
		{
			allocsize+=SAVE_BUFFSIZE;
			if((h=HeapRealloc(h, allocsize))==H_NULL)
				return FALSE;
			ptrout = HeapDeref(h);
		}
		
	memcpy(ptrout+writepos,dbtag,sizeof(dbtag));
	writepos+=sizeof(dbtag)-2;
	
//write the length of the database
	*(unsigned int*)(ptrout)=writepos;
	
	HeapRealloc(h, writepos+2);
	HeapUnlock(h);
	
	{
		char buff[19];
		buff[0]=0;
		strcpy(buff+1,gdbname);	
		if(DerefSym(SymFind(buff+strlen(gdbname)+1))->flags.bits.archived)
		{
			dlg_help(WARN_ARCHIVED);
			HeapFree(h);
		}
		else
		{
			sym=SymAdd(buff+strlen(gdbname)+1);
			DerefSym(sym)->handle=h;
		}
	}
	
	return TRUE;
}

int MenuNewDataBase(void)
{
	volatile HANDLE *hACT = &ghACT;
	volatile HANDLE *hALIAS = &ghALIAS;
	volatile HANDLE *hCUST = &ghCUST;
	
	char *dbtag[]={"I DB",NULL};
	unsigned char dbtag0[]={0xF8,0};
	HSym sym;
	
//display the New file dialog box
	if(strlen(gdbname))
	{
		sym=VarNew(dbtag0,dbtag);
		if(sym.folder==0)
			return FALSE;
		HSYMtoName(sym, gdbname);
		HSymDel(sym);
	}
	else
	{
		strcpy(gdbname,"main\\ashdat");
	}
	
//allocate or reallocate space for the text editors
	if(!*hCUST)*hCUST=HeapRealloc(*hCUST, 8);
	if(!*hALIAS)*hALIAS=HeapRealloc(*hALIAS, 8);
	if(!*hACT)*hACT=HeapRealloc(*hACT, 8);

//Null Terminate the buffers
	*(char*)HeapDeref(*hCUST)=0;
	*(char*)HeapDeref(*hALIAS)=0;
	*(char*)HeapDeref(*hACT)=0;

//update the screen
	ChangeWindow(0);
	
	return TRUE;
}
